/*
 * Name:    mxbl.h
 *
 * Purpose: Header for the MX Blu-Ice server.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2008 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __MXBL_H__
#define __MXBL_H__

#include "mx_bluice.h"

typedef struct {
	MX_LIST *event_list;
	int max_fd;
	mx_bool_type dhs_socket_is_present;
	char dhs_name[MXU_BLUICE_NAME_LENGTH+1];
	struct mxbl_event_type *master;
	unsigned long operation_number;
	MX_RECORD *record_list;
} MXBL_EVENT_MANAGER;

typedef struct mxbl_event_type {
	MX_SOCKET *mx_socket;
	long socket_type;
	void (*event_handler)(struct mxbl_event_type *);
	MXBL_EVENT_MANAGER *event_manager;
	mx_bool_type protocol_initialized;
	unsigned long client_number;
	char message_buffer[MX_BLUICE_OLD_MESSAGE_LENGTH+2];
} MXBL_EVENT;

typedef struct {
	MXBL_EVENT_MANAGER *event_manager;
	MX_RECORD *record;
	mx_bool_type operation_in_progress;
	char bluice_name[MXU_BLUICE_NAME_LENGTH+1];

	void *rinfo_private;
} MXBL_RECORD_INFO;

typedef struct {
	long num_ion_chambers;
	MX_RECORD **ion_chamber_array;
} MXBL_TIMER_INFO;

typedef struct {
	char run_status[MXU_BLUICE_NAME_LENGTH+1];
	unsigned long next_frame;
	unsigned long run_label;
	char file_root[MXU_FILENAME_LENGTH+1];
	char directory[MXU_FILENAME_LENGTH+1];
	unsigned long start_frame_label;
} MXBL_SCAN_INFO;

typedef struct {
	MXBL_EVENT *event;
	char operation_name[40];
	unsigned long client_number;
	unsigned long operation_number;
	mx_bool_type operation_in_progress;

	char *start_message;

	void *op_private;
} MXBL_OPERATION_ARGS;

/* FIXME: Get rid of this global variable! */

extern MXBL_OPERATION_ARGS *global_detector_collect_image_args;

/* Values for 'socket_type' */

#define MXBLT_DHS_SOCKET		1
#define MXBLT_DCSS_SERVER_SOCKET	2
#define MXBLT_DCSS_CLIENT_SOCKET	3

/*------*/

extern mx_status_type mxbl_create_event_manager( MXBL_EVENT_MANAGER **manager,
						MX_RECORD *record_list,
						char *dhs_name );

extern mx_status_type mxbl_event_manager_register_socket_event(
				MXBL_EVENT_MANAGER *manager,
				MX_SOCKET *mx_socket,
				long socket_type,
				void (*event_handler)(MXBL_EVENT *),
				MXBL_EVENT **new_event );

extern mx_status_type mxbl_event_manager_unregister_socket_event(
				MXBL_EVENT *event );

extern mx_status_type mxbl_event_manager_process_events(
				MXBL_EVENT_MANAGER *manager,
				double timeout_in_seconds );

extern mx_status_type mxbl_event_manager_list_socket_events(
				MXBL_EVENT_MANAGER *manager,
				char *label );

/*---------*/

extern mx_status_type mxbl_register_dcss_callbacks(
					MXBL_EVENT_MANAGER *event_manager,
					MX_RECORD *record_list );

extern mx_status_type mxbl_setup_record_info(
					MXBL_EVENT_MANAGER *event_manager,
					MX_RECORD *record,
					char *bluice_name );

extern mx_status_type mxbl_register_field_callback( MX_RECORD *record,
							char *field_name );

extern mx_status_type mxbl_record_field_callback( MX_CALLBACK *callback,
							void *args );

/*---------*/

extern mx_status_type mxbl_send_message( MX_SOCKET *mx_socket,
					char *text_data,
					char *binary_data,
					long binary_data_length,
					long protocol_version );

extern mx_status_type mxbl_receive_message( MX_SOCKET *mx_socket,
					char *data_buffer,
					long data_buffer_length,
					long *actual_data_length,
					double timeout_in_seconds,
					long protocol_version );

/*---------*/

extern void mxbl_handle_dcss_server_socket( MXBL_EVENT *event );

extern void mxbl_handle_dcss_messages( MXBL_EVENT *event );

extern void mxbl_handle_dhs_messages( MXBL_EVENT *event );

extern mx_status_type mxbl_broadcast_dcss_message(
					MXBL_EVENT_MANAGER *event_manager,
					char *message_buffer );
/*---------*/

extern mx_status_type mxbl_find_ion_chambers( MX_RECORD *timer_record );

extern mx_status_type mxbl_setup_scan_info( MX_RECORD *scan_record );

extern mx_status_type mxbl_set_string( MXBL_EVENT_MANAGER *event_manager,
					MX_RECORD *string_record,
					char *string_value );

extern mx_status_type mxbl_set_collect_msg( MXBL_EVENT_MANAGER *event_manager,
						char *string_value );

extern mx_status_type mxblh_abort_all( MXBL_EVENT *event );

extern mx_status_type mxblh_read_ion_chambers( MXBL_EVENT *event );

extern mx_status_type mxblh_set_shutter_state( MXBL_EVENT *event );

extern mx_status_type mxblh_start_motor_move( MXBL_EVENT *event );

extern mx_status_type mxblh_start_operation( MXBL_EVENT *event );

/*---------*/

extern mx_status_type mxbl_tcl_string_split( char *original_string,
					int *argc, char ***argv );

#endif /* __MXBL_H__ */

